#ifndef CORE_CRC32_H
#define CORE_CRC32_H

namespace rde
{
// Class representing CRC32 checksum value
class CRC32
{
public:
	// Giev C99!
	typedef unsigned char	uint8_t;
	typedef unsigned short	uint16_t;
	typedef unsigned long	uint32_t;

	CRC32();

	void Add(uint8_t);
	void Add(uint16_t);
	void Add(uint32_t);
	// Null terminated string.
	void AddString(const char* str);
	void AddArray(const uint8_t* bytes, int numBytes);

	void Reset();
	uint32_t GetValue() const	{ return m_value; }
	
private:
	uint32_t	m_value;
};

inline bool operator==(const CRC32& lhs, const CRC32& rhs)
{
	return lhs.GetValue() == rhs.GetValue();
}
inline bool operator!=(const CRC32& lhs, const CRC32& rhs)
{
	return !(lhs == rhs);
}
inline bool operator<(const CRC32& lhs, const CRC32& rhs)
{
	return lhs.GetValue() < rhs.GetValue();
}

} // rde

#endif 
