#include "sqlite/SQLDatabase.h"
#include "sqlite/sqlite3.h"
#include <cassert>
#include <cstdio>

namespace sql3
{
const int Database::OPEN_READONLY		= SQLITE_OPEN_READONLY;
const int Database::OPEN_READWRITE		= SQLITE_OPEN_READWRITE;
const int Database::OPEN_CREATE		= SQLITE_OPEN_CREATE;
const int Database::OPEN_DEFAULT		= SQLITE_OPEN_READWRITE | SQLITE_OPEN_CREATE;

Database::Database()
:	m_db(0)
{
	/**/
}

Database::~Database()
{
	Disconnect();
}

bool Database::Connect(const char* filename, int flags)
{
	assert(flags != 0x0);
	Disconnect();
	const int err = ::sqlite3_open_v2(filename, &m_db, flags, 0);
	return err == SQLITE_OK && m_db != 0;
}

bool Database::Disconnect()
{
	bool ok(true);
	if (m_db != 0)
		ok = ::sqlite3_close(m_db) == SQLITE_OK;
	m_db = 0;
	return ok;
}

bool Database::Execute(const char* cmd) 
{
	return ::sqlite3_exec(m_db, cmd, 0, 0, 0) == SQLITE_OK;
}

bool Database::Executef(const char* cmd, ...)
{
	static const int MAX_CMD_SIZE	= 512;
	va_list args;
	va_start(args, cmd); 
	char buff[MAX_CMD_SIZE];
	const int size = _vsnprintf(buff, MAX_CMD_SIZE, cmd, args);
	assert(size >= 0);
	(void)size;
	va_end(args);
	return Execute(buff);
}

const char* Database::GetLastErrorMessage() const
{
	return ::sqlite3_errmsg(m_db);
}

} // sql3

