#include "sqlite/SQLQuery.h"
#include "sqlite/sqlite3.h"
#include "sqlite/SQLDatabase.h"
#include <cassert>
#include <cstdio>
#include <cstring>

namespace
{
const int MAX_CMD_SIZE	= 512;
}

namespace sql3
{
Query::Query(Database& db)
:	m_db(db),
	m_statement(0),
	m_rowValid(false)
{
	/**/
}

Query::~Query()
{
	Finalize();
}

bool Query::Execute(const char* cmd)
{
	Finalize();

	const char* tail(0);
	const int ret = ::sqlite3_prepare(m_db.m_db, cmd, (int)strlen(cmd), &m_statement, &tail);
	if (ret != SQLITE_OK || m_statement == 0)
		return false;

	return true;
}

bool Query::Executef(const char* cmd, ...)
{
	va_list args;
	va_start(args, cmd); 
	char buff[MAX_CMD_SIZE];
	const int size = _vsnprintf(buff, MAX_CMD_SIZE, cmd, args);
	assert(size >= 0);
	(void)size;
	va_end(args);
	return Execute(buff);
}

int Query::NextRow()
{
	m_rowValid = false;
	if (m_statement)
	{
		const int ret = ::sqlite3_step(m_statement);
		if (ret == SQLITE_ROW)
		{
			m_rowValid = true;
			return 1;
		}
		return ret == SQLITE_DONE ? 0 : -1;
	}
	return 0;
}

int Query::NumColumns() const
{
	return m_statement ? sqlite3_column_count(m_statement) : 0;
}

const char* Query::GetString(int columnIndex) const
{
	assert(IsIteratorValid() && columnIndex >= 0 && columnIndex < NumColumns());
	return (const char*)::sqlite3_column_text(m_statement, columnIndex);
}

int Query::GetInt(int columnIndex) const
{
	assert(IsIteratorValid() && columnIndex >= 0 && columnIndex < NumColumns());
	return ::sqlite3_column_int(m_statement, columnIndex);
}

double Query::GetDouble(int columnIndex) const
{
	assert(IsIteratorValid() && columnIndex >= 0 && columnIndex < NumColumns());
	return sqlite3_column_double(m_statement, columnIndex);
}

void Query::Finalize()
{
	if (m_statement)
		::sqlite3_finalize(m_statement);
	m_statement = 0;
	m_rowValid = false;
}

bool Query::IsIteratorValid() const
{
	return m_statement && m_rowValid;
}

}
