#ifndef SQLITE_SQLQUERY_H
#define SQLITE_SQLQUERY_H

struct sqlite3_stmt;

namespace sql3
{
class Database;

// Typical query cycle:
//	Query q(db);
//	q.Execute("SELECT * from TAB")
//	while (q.NextRow() > 0)
//		// fetch row using GetString/Int/Double
//
class Query
{
public:
	explicit Query(Database& db);
	~Query();

	bool Execute(const char* cmd);
	bool Executef(const char* cmd, ...);

	// 0 - end of iteration/1 - success, -1 - error
	int NextRow();
	int NumColumns() const;
	const char* GetString(int columnIndex) const;
	int GetInt(int columnIndex) const;
	double GetDouble(int columnIndex) const;
	void Finalize();

	bool IsIteratorValid() const;

private:
	Database&		m_db;
	sqlite3_stmt*	m_statement;
	bool			m_rowValid;
};

}

#endif 
