#include "sqlite/SQLDatabase.h"
#include "sqlite/SQLQuery.h"
#include "sqlite/CRC32.h"
#include <cassert>
#include <cstdio>
#include <windows.h>

rde::CRC32 GetCRC32(const char* str, sql3::Database& db)
{
	rde::CRC32 crc;
	crc.AddString(str);

	// If such CRC exists in the database -- we have to make sure it's our string,
	// otherwise - we have a conflict.
	sql3::Query q(db);
	bool ok = q.Executef("SELECT * FROM crcTab WHERE crc=%u", crc.GetValue());
	assert(ok);
	const int rc = q.NextRow();
	assert(rc >= 0);
	if (rc == 1)
	{
		const char* strInBase = q.GetString(1);
		// @note:	In "real world" usage you'll most probably want
		//			case insensitive version of CRC32.
		assert(strcmp(str, strInBase) == 0);
		(void)strInBase;
	}
	else if (rc == 0) // If it doesnt exist yet -- add it.
	{
		ok = db.Executef("INSERT INTO crcTab VALUES ('%u', '%s')", crc.GetValue(), str);
		assert(ok);
	}
	return crc;
}

void EnumerateFiles(sql3::Database& db)
{
	WIN32_FIND_DATA ffd;
	const HANDLE hFind = FindFirstFile("c:\\Windows\\System32\\*.*", &ffd);
	if (hFind)
	{
		do
		{
			if (!(ffd.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY))
			{
				/*const rde::CRC32 crc =*/GetCRC32(ffd.cFileName, db);
			}
		} while (FindNextFile(hFind, &ffd) != 0);
		FindClose(hFind);
	}
}

int main()
{
	sql3::Database db;
	if (!db.Connect("c:\\crc32.db"))
	{
		::printf("Cannot open CRC32 database.");
		return 1;
	}
	EnumerateFiles(db);
	return 0;
}
		